local mod = ...



-- Building Asset Processor -------------------------------------------------------------------------------------
mod:registerAssetProcessor("models/UndergroundMine1.fbx", { DataType = "BUILDING_ASSET_PROCESSOR" })



-- Building -------------------------------------------------------------------------------------
mod:registerAsset({
	DataType = "BUILDING",
	Id = "BUILDING_UNDERGROUNDMINE",
	Name = "BUILDING_UNDERGROUNDMINE_NAME",
	Description = "BUILDING_UNDERGROUNDMINE_DESC",
	BuildingType = "RESOURCE_PRODUCTION",
	AssetCoreBuildingPart = "BUILDING_PART_MONUMENT_POLE",
	BuildingPartSetList = {
		{
			Name = "BUILDING_PART_SETLIST_UNDERGROUNDMINE1_NAME",
			BuildingPartList = {
				"BUILDING_PART_UNDERGROUNDMINE_MINE",
				"BUILDING_PART_UNDERGROUNDMINE_DEPOT",
			},
        },
    },
    VillagerRequired = { Status = "NEWCOMER", Quantity = 20 },
    RequiredPartList = {
        {
            Category = "QUARRY",
            Min = 1,
            Max = 1,
        },
        {
            Category = "RESOURCE_DEPOT",
            Min = 1,
            Max = 1,
        },
    }
})



-- Resource: Mining Spoil -------------------------------------------------------------------------------------
mod:registerEnumValue( "RESOURCE_TYPE", "RESOURCE_TYPE_UNDERGROUNDMINE" )

mod:registerAsset({
	DataType = "RESOURCE",
	Id = "RESOURCE_MINE_SPOIL",
	ResourceName = "RESOURCE_MINE_SPOIL_NAME",
	Icon = "ICON_RESOURCE_COAL",
	ResourceTypeList = { "RESOURCE_TYPE_UNDERGROUNDMINE" },
	IsTradable = false,
    DisplayInInventory = false,
    DisplayInToolbar = false,
	ResourceLayoutType = "CRATES",
    OrderId = 811,
})



-- Job: Miner -------------------------------------------------------------------------------------
mod:registerAsset({
	DataType = "JOB",
	Id = "JOB_UNDERGROUNDMINE_MINE",
	JobName = "JOB_UNDERGROUNDMINE_MINE_NAME",
	JobDescription = "JOB_UNDERGROUNDMINE_MINE_DESC",
	IsLockedByDefault = false,
	DefaultBehavior = "BEHAVIOR_WORK",
    ProductionDelay = 60, -- default 30
	AssetJobProgression = "DEFAULT_JOB_PROGRESSION",
	CharacterSetup = {
		DataType = "CHARACTER_SETUP",
        RightHandObject = {
			Model = "PREFAB_TOOL_STONECUTTER_PICKAXE"
        },
		IdleAnimation = "MINE",
	},
})

mod:overrideAsset({
    Id = "NEWCOMER",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_UNDERGROUNDMINE_MINE"
    }
})

mod:overrideAsset({
    Id = "SERF",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_UNDERGROUNDMINE_MINE"
    }
})



-- Job: Depot -------------------------------------------------------------------------------------
mod:registerAsset({
	DataType = "JOB",
	Id = "JOB_UNDERGROUNDMINE_DEPOT",
	JobName = "JOB_UNDERGROUNDMINE_DEPOT_NAME",
	JobDescription = "JOB_UNDERGROUNDMINE_DEPOT_DESC",
	IsLockedByDefault = false,
	DefaultBehavior = "BEHAVIOR_WORK",
    ProductionDelay = 30, -- default 30
	AssetJobProgression = "DEFAULT_JOB_PROGRESSION",
	CharacterSetup = {
		DataType = "CHARACTER_SETUP",
        RightHandObject = {
			Model = "PREFAB_TOOL_COAL_BURNER_SHOVEL"
        },
		IdleAnimation = "GOLDSMELTER_SHOVELING",
	},
})

mod:overrideAsset({
    Id = "NEWCOMER",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_UNDERGROUNDMINE_DEPOT"
    }
})

mod:overrideAsset({
    Id = "SERF",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_UNDERGROUNDMINE_DEPOT"
    }
})



-- Building part: Mine ------------------------------------------------------------------------------------
mod:registerAssetId("models/UndergroundMine1.fbx/Prefab/Mine.Part.1", "PREFAB_UNDERGROUNDMINE_MINE")
mod:registerAssetId("models/UndergroundMine1.fbx/Prefab/Mine.Part.1.Constr", "PREFAB_UNDERGROUNDMINE_MINE_CONSTR")
mod:configurePrefabFlagList("PREFAB_UNDERGROUNDMINE_MINE", { "PLATFORM" })

-- Function
mod:registerAsset({
    DataType = "BUILDING_FUNCTION_QUARRY",
    Id = "BUILDING_FUNCTION_UNDERGROUNDMINE_MINE",
    Name = "BUILDING_FUNCTION_UNDERGROUNDMINE_MINE_NAME",
    WorkerCapacity = 4,
    RelatedJob = {
        Job = "JOB_UNDERGROUNDMINE_MINE",
        Behavior = "BEHAVIOR_WORK"
    },
    UpkeepPerCapacity = {
        { Resource = "GOLD_COINS", Quantity = 2 },
    },
    InputInventoryCapacity = {
        { Resource = "WOOD", Quantity = 20 },
        { Resource = "PLANK", Quantity = 20 },
        { Resource = "CANDLE", Quantity = 20 },
    },
    ResourceListNeeded = {
        { Resource = "WOOD", Quantity = 1 },
        { Resource = "PLANK", Quantity = 1 },
        { Resource = "CANDLE", Quantity = 1 },
    },
    ResourceProduced = {
        { Resource = "RESOURCE_MINE_SPOIL", Quantity = 2 },
    },
    RandomResourceToSpawn = "IRON_ORE",
    SpawnPercentage = 0.15,
})

-- Building part
mod:registerAsset({
    DataType = "BUILDING_PART",
    Id = "BUILDING_PART_UNDERGROUNDMINE_MINE_UNDERGROUND",
    Name = "BUILDING_PART_UNDERGROUNDMINE_MINE_NAME",
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
        -- CoreObjectPrefab = "PREFAB_MINERAL_SOURCE_1",
        CoreObjectPrefab = "PREFAB_RESOURCE_VISUAL_COAL_CRATE",
    },
})

mod:registerAsset({
    DataType = "BUILDING_PART",
    Id = "BUILDING_PART_UNDERGROUNDMINE_MINE_ASSEMBLAGE",
    Name = "BUILDING_PART_UNDERGROUNDMINE_MINE_NAME",
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_ASSEMBLAGE",
        CoreRandomBuildingPartList = { "PREFAB_UNDERGROUNDMINE_MINE" },
        MandatoryBuildingPartList = {
            { BuildingPart = "BUILDING_PART_UNDERGROUNDMINE_MINE_UNDERGROUND", OptionalAttachNodeString = "Attach_Major.001" },
        },
    },
})


mod:registerAsset({
    DataType = "BUILDING_PART",
    Id = "BUILDING_PART_UNDERGROUNDMINE_MINE",
    Name = "BUILDING_PART_UNDERGROUNDMINE_MINE_NAME",
    Category = "QUARRY",
    AssetBuildingFunction = "BUILDING_FUNCTION_UNDERGROUNDMINE_MINE",
    ConstructionVisual = "PREFAB_UNDERGROUNDMINE_MINE_CONSTR",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_RANDOM_PART",
		RandomPartList = {
			"BUILDING_PART_UNDERGROUNDMINE_MINE_ASSEMBLAGE",
		},
        MiniatureConfig = {
            ClipUnderGround = true,
            GroupHeight = 0.0,
            CameraPosition = { 0, 2.25, -21 },
            OrientationOffset = -45.0,
            CameraPitchOffset = 0.0
        },
	},
    BuildingZone = {
        ZoneEntryList = {
            {
                Polygon = polygon.createCircle( 5.0, {0, 0}, 8 ),
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = false }
            },
        }
    },
    Cost = {
    	BuildRightTaxes = { { Resource = "GOLD_COINS", Quantity = 60 } },
    	UpkeepCost = { { Resource = "GOLD_COINS", Quantity = 20 } },
    	ResourceNeededList = {
            {
                { Resource = "WOOD", Quantity = 10 },
                { Resource = "PLANK", Quantity = 20 },
                { Resource = "TOOL", Quantity = 10 },
            },
    	},
    },
})



-- Depot -------------------------------------------------------------------------------------
mod:registerAssetId("models/UndergroundMine1.fbx/Prefab/Depot.Part.1", "PREFAB_UNDERGROUNDMINE_DEPOT")
mod:registerAssetId("models/UndergroundMine1.fbx/Prefab/Depot.Part.1.Constr", "PREFAB_UNDERGROUNDMINE_DEPOT_CONSTR")
mod:registerAssetId("models/UndergroundMine1.fbx/Prefab/Depot.Part.1/Deposit.Mineral", "PREFAB_UNDERGROUNDMINE_DEPOT_DEPOSIT1")
mod:registerAssetId("models/UndergroundMine1.fbx/Prefab/Depot.Part.1/Deposit.Gems", "PREFAB_UNDERGROUNDMINE_DEPOT_DEPOSIT2")
mod:registerPrefabComponent("models/UndergroundMine1.fbx/Prefab/Depot.Part.1/Wheelbarrow.001", { DataType = "COMP_GROUNDED" } )
mod:registerPrefabComponent("models/UndergroundMine1.fbx/Prefab/Depot.Part.1/Crane.001", { DataType = "COMP_GROUNDED" } )
mod:registerAssetId("models/UndergroundMine1.fbx/Prefab/Blank.Part", "PREFAB_UNDERGROUNDMINE_BLANK")


-- Function
mod:registerAsset({
    DataType = "BUILDING_FUNCTION_UNIQUE_RESOURCE_DEPOT",
    Id = "BUILDING_FUNCTION_UNDERGROUNDMINE_DEPOT",
    Name = "BUILDING_FUNCTION_UNDERGROUNDMINE_DEPOT_NAME",
    MineralStockpilePrefabList = {
        "PREFAB_RESOURCE_ROCK",
    },
    MineralStockpileLocator = "PREFAB_UNDERGROUNDMINE_DEPOT_DEPOSIT1",
    MineralCapacity = 100,
    GemsStockpilePrefab = "PREFAB_RESOURCE_ROCK",
    GemsStockpileLocator = "PREFAB_UNDERGROUNDMINE_DEPOT_DEPOSIT2",
    GemsResourceVisualPrefab = "PREFAB_RESOURCE_VISUAL_STONE_CRATE",
    GemsCapacity = 50,
})

mod:registerAsset({
    DataType = "BUILDING_FUNCTION_WORKPLACE",
    Id = "BUILDING_FUNCTION_UNDERGROUNDMINE_DESTROY",
    Name = "BUILDING_FUNCTION_UNDERGROUNDMINE_DEPOT_NAME",
    WorkerCapacity = 1,
    RelatedJob = {
        Job = "JOB_UNDERGROUNDMINE_DEPOT",
        Behavior = "BEHAVIOR_WORK"
    },
    UpkeepPerCapacity = {
        { Resource = "GOLD_COINS", Quantity = 2 },
    },
    InputInventoryCapacity = {
        { Resource = "RESOURCE_MINE_SPOIL", Quantity = 50 },
    },
    ResourceListNeeded = {
        { Resource = "RESOURCE_MINE_SPOIL", Quantity = 10 },
    },
    ResourceProduced = {
        { Resource = "RESOURCE_MINE_SPOIL", Quantity = 0 },
    },
})

-- Building part
mod:registerAsset({
    DataType = "BUILDING_PART",
    Id = "BUILDING_PART_UNDERGROUNDMINE_DEPOT_BLANK",
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
        CoreObjectPrefab = "PREFAB_UNDERGROUNDMINE_BLANK",
    },
    AssetBuildingFunction = "BUILDING_FUNCTION_UNDERGROUNDMINE_DEPOT",
})

mod:registerAsset({
    DataType = "BUILDING_PART",
    Id = "BUILDING_PART_UNDERGROUNDMINE_MINE_DEPOT_ASSEMBLAGE",
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_ASSEMBLAGE",
        CoreRandomBuildingPartList = { "PREFAB_UNDERGROUNDMINE_DEPOT" },
        MandatoryBuildingPartList = {
            { BuildingPart = "BUILDING_PART_UNDERGROUNDMINE_DEPOT_BLANK" },
        },
    },
    ConstructionVisual = "PREFAB_UNDERGROUNDMINE_DEPOT_CONSTR",
})

mod:registerAsset({
    DataType = "BUILDING_PART",
    Id = "BUILDING_PART_UNDERGROUNDMINE_DEPOT",
    Name = "BUILDING_PART_UNDERGROUNDMINE_DEPOT_NAME",
    Category = "RESOURCE_DEPOT",
    AssetBuildingFunction = "BUILDING_FUNCTION_UNDERGROUNDMINE_DESTROY",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_RANDOM_PART",
		RandomPartList = {
			"BUILDING_PART_UNDERGROUNDMINE_MINE_DEPOT_ASSEMBLAGE",
		}
	},
    BuildingZone = {
        ZoneEntryList = {
            {
                Polygon = polygon.createCircle( 3.0, {0, 0}, 8 ),
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = false }
            },
        }
    },
    Cost = {
    	ResourceNeededList = {
            {
                { Resource = "WOOD", Quantity = 4 },
                { Resource = "PLANK", Quantity = 4 },
                { Resource = "TOOL", Quantity = 4 },
            },
    	},
    },
})



